"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var extractor_utils_1 = require("../extractor-utils");
var types_1 = require("@babel/types");
var notParcelBundleErr = function () { return new TypeError("input is not a parcel bundle AST"); };
exports.extractModules = function (ast, options) {
    ast = extractor_utils_1.getInputProgramAST(ast);
    // check parcel bundle
    var parcelRequireES = ast.body[0]; // parcelRequire=xxx
    if (!types_1.isExpressionStatement(parcelRequireES)) {
        throw notParcelBundleErr();
    }
    var parcelRequireE = parcelRequireES.expression;
    // https://github.com/parcel-bundler/parcel/blob/fe0066b/packages/core/parcel-bundler/src/builtins/prelude.js
    if (!types_1.isAssignmentExpression(parcelRequireE) || !types_1.isIdentifier(parcelRequireE.left) || !types_1.isCallExpression(parcelRequireE.right)) {
        throw notParcelBundleErr();
    }
    if (parcelRequireE.left.name !== "parcelRequire") {
        throw notParcelBundleErr();
    }
    // https://github.com/parcel-bundler/parcel/blob/c2760fd/packages/core/parcel-bundler/src/packagers/JSPackager.js#L229
    var _a = __read(parcelRequireE.right.arguments, 3), parcelModulesObj = _a[0], entryA = _a[2];
    // get entry asset id
    if (!types_1.isArrayExpression(entryA)) {
        throw notParcelBundleErr();
    }
    var entryAssetIdE = entryA.elements.pop();
    var entryAssetId = entryAssetIdE.value;
    // override
    if (options && options.entryPoint) {
        entryAssetId = options.entryPoint;
    }
    // extract modules
    if (!types_1.isObjectExpression(parcelModulesObj)) {
        throw notParcelBundleErr();
    }
    var parcelModules = parcelModulesObj.properties.map(function (p) {
        if (!types_1.isObjectProperty(p)) {
            return;
        }
        // type AssetId = string | number
        if (!(types_1.isStringLiteral(p.key) || types_1.isNumericLiteral(p.key))) {
            throw notParcelBundleErr();
        }
        var id = p.key.value;
        var isEntry = id == entryAssetId;
        // modules are defined as an array: [ module function, map of requires ]
        if (!types_1.isArrayExpression(p.value)) {
            throw notParcelBundleErr();
        }
        var _a = __read(p.value.elements, 2), moduleFunction = _a[0], requires = _a[1];
        // get module ast block
        // function(require,module,exports) { …
        if (!types_1.isFunctionExpression(moduleFunction)) {
            throw notParcelBundleErr();
        }
        if (moduleFunction.params.length !== 3 || !moduleFunction.params.every(function (x) { return types_1.isIdentifier(x); })) {
            throw notParcelBundleErr();
        }
        var moduleAST = moduleFunction.body;
        var moduleRequires = {};
        if (!types_1.isObjectExpression(requires)) {
            throw notParcelBundleErr();
        }
        requires.properties.forEach(function (r) {
            if (!types_1.isObjectProperty(r)) {
                throw notParcelBundleErr();
            }
            if (!(types_1.isStringLiteral(r.value) || types_1.isNumericLiteral(r.value))) {
                throw notParcelBundleErr();
            }
            if (!types_1.isStringLiteral(r.key)) {
                throw notParcelBundleErr();
            }
            moduleRequires[r.key.value] = r.value.value;
        });
        return {
            id: id,
            ast: moduleAST,
            requires: moduleRequires,
            isEntry: isEntry,
        };
    });
    var parcelModulesMap = new Map(parcelModules.map(function (m) { return [m.id, m]; }));
    return {
        entry: entryAssetId,
        modulesMap: parcelModulesMap,
    };
};
exports.default = exports.extractModules;
