"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var extract_modules_1 = require("./extract-modules");
var types_1 = require("@babel/types");
var entrypointNotfoundErr = function () { return new Error("options.entryPoint is undefined and failed to get entry id."); };
exports.getWebpackBundleInfo = function (callAST, options) {
    var callee = callAST.callee, modulesAST = callAST.arguments[0];
    if (types_1.isIdentifier(callee)) {
        throw new TypeError("This bundle looks like a webpack-jsonp bundle.\nset options.type = 'webpack-jsonp', and try again.");
    }
    else if (!types_1.isFunctionExpression(callee)) {
        throw extract_modules_1.notWebpackBootstrapAstErr();
    }
    // try to get entry id
    if (typeof options.entryPoint !== "string" || typeof options.entryPoint !== "number") {
        options.entryPoint = undefined;
        try {
            // unminified file
            // the last statement:
            // e.g. `return __webpack_require__(__webpack_require__.s = 8);`
            // minified file
            // the last statement:
            // e.g. `return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s=8)`
            var lastStatement = callee.body.body.pop();
            if (!types_1.isReturnStatement(lastStatement)) {
                throw new Error();
            }
            var e = lastStatement.argument;
            var entryRequireCall = void 0;
            if (types_1.isCallExpression(e)) {
                entryRequireCall = e;
            }
            else if (types_1.isSequenceExpression(e)) {
                entryRequireCall = e.expressions.pop();
            }
            if (!types_1.isCallExpression(entryRequireCall)) {
                throw new Error();
            }
            var entryRequireCallArgument = entryRequireCall.arguments[0];
            if (types_1.isNumericLiteral(entryRequireCallArgument)) {
                options.entryPoint = entryRequireCallArgument.value;
            }
            else if (types_1.isAssignmentExpression(entryRequireCallArgument)) {
                var a = entryRequireCallArgument.right;
                if (types_1.isNumericLiteral(a)) {
                    options.entryPoint = a.value;
                }
            }
            if (typeof options.entryPoint == "undefined") {
                throw new Error();
            }
        }
        catch (_) {
            throw entrypointNotfoundErr();
        }
    }
    return {
        entryId: options.entryPoint,
        modulesAST: modulesAST,
    };
};
exports.getWebpackJsonpBundleInfo = function (callAST, options) {
    var callee = callAST.callee, callArgs = callAST.arguments;
    if (types_1.isFunctionExpression(callee)) {
        throw new TypeError("This bundle looks like a normal webpack bundle.\nset options.type = 'webpack', and try again.");
    }
    var callArguments = callArgs.length == 1
        ? callArgs[0].elements
        : callArgs;
    // const [selfModuleIdArrE, modulesAST, entryIdArrE] = callArguments
    var _a = __read(callArguments, 3), modulesAST = _a[1], entryIdArrE = _a[2];
    // try to get entry id
    if (typeof options.entryPoint !== "string" || typeof options.entryPoint !== "number") {
        options.entryPoint = undefined;
        try {
            if (!types_1.isArrayExpression(entryIdArrE)) {
                throw new Error();
            }
            var _b = __read(entryIdArrE.elements, 1), entryIdE = _b[0];
            if (!entryIdE || !types_1.isNumericLiteral(entryIdE)) {
                throw new Error();
            }
            options.entryPoint = entryIdE.value;
        }
        catch (_) {
            // throw ENTRYPOINT_NOTFOUND_ERR
        }
    }
    return {
        entryId: options.entryPoint,
        modulesAST: modulesAST,
    };
};
