"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
var extractor_utils_1 = require("../extractor-utils");
var bundle_info_1 = require("./bundle-info");
var types_1 = require("@babel/types");
exports.notWebpackBootstrapAstErr = function () { return new TypeError("not a webpackBootstrap function call AST."); };
var moduleFunctionParams = ["module", "exports", "__webpack_require__"];
exports.extractModules = function (ast, options) {
    var e_1, _a;
    ast = extractor_utils_1.getInputProgramAST(ast);
    if (Array.isArray(options.bundleAstReferenceKeys)) {
        try {
            for (var _b = __values(options.bundleAstReferenceKeys), _c = _b.next(); !_c.done; _c = _b.next()) {
                var k = _c.value;
                ast = ast[k];
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    else {
        throw new TypeError("options.moduleAstEntryKeys is not an array.");
    }
    if (!types_1.isCallExpression(ast)) {
        throw exports.notWebpackBootstrapAstErr();
    }
    var bundleInfo;
    if (options.type == "webpack-jsonp") {
        bundleInfo = bundle_info_1.getWebpackJsonpBundleInfo(ast, options);
    }
    else {
        // if (options.type == "webpack") {
        bundleInfo = bundle_info_1.getWebpackBundleInfo(ast, options);
    }
    var entryId = bundleInfo.entryId, modulesAST = bundleInfo.modulesAST;
    var ext = options.outputFileType == "typescript" ? "ts" : "js";
    var entryPath = "entry_" + entryId + "." + ext;
    var modules = {};
    var solveModule = function (moduleFunction, id) {
        var isEntry = id == entryId;
        // an empty module funtion in a modulesAST (ArrayExpression), e.g. [,,, normal module funtion ]
        if (!moduleFunction) {
            return;
        }
        // get module ast block
        if (!types_1.isFunctionExpression(moduleFunction)) {
            throw exports.notWebpackBootstrapAstErr();
        }
        var params = moduleFunction.params, moduleAST = moduleFunction.body;
        var modulePath = "" + (isEntry ? "entry_" : "") + id + "." + ext;
        // put module function params in module body 
        // minified `function(e,t,n){`
        // unminified `function(module, exports, __webpack_require__) {`
        // -> `const e = module, t = exports, n = __webpack_require__`
        // -> `const e = module; const t = exports; const n = __webpack_require__;` (by transforms/unminify-variable-declarations.ts)
        moduleAST.body.unshift(types_1.variableDeclaration("const", params.map(function (p, index) {
            if (!types_1.isIdentifier(p)) {
                throw exports.notWebpackBootstrapAstErr();
            }
            return types_1.variableDeclarator(types_1.identifier(p.name), types_1.identifier(moduleFunctionParams[index]));
        })));
        modules[modulePath] = {
            id: id,
            ast: moduleAST,
            isEntry: isEntry,
        };
    };
    if (types_1.isArrayExpression(modulesAST)) {
        modulesAST.elements.forEach(solveModule);
    }
    else if (types_1.isObjectExpression(modulesAST)) {
        modulesAST.properties.forEach(function (p) {
            if (!types_1.isObjectProperty(p)) {
                throw exports.notWebpackBootstrapAstErr();
            }
            if (!types_1.isStringLiteral(p.key) && !types_1.isNumericLiteral(p.key) && !types_1.isIdentifier(p.key)) {
                throw exports.notWebpackBootstrapAstErr();
            }
            if (!types_1.isFunctionExpression(p.value)) {
                throw exports.notWebpackBootstrapAstErr();
            }
            solveModule(p.value, p.key.name || p.key.value);
        });
    }
    else {
        throw exports.notWebpackBootstrapAstErr();
    }
    return {
        modules: modules,
        entry: entryId,
        entryPath: entryPath,
    };
};
exports.default = exports.extractModules;
