"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var visitor_wrapper_1 = require("../../utils/visitor-wrapper");
var parser_1 = require("@babel/parser");
var types_1 = require("@babel/types");
var transformErr = function () { return new Error("something goes wrong."); };
var unknownRuntimeFnErr = function () { return new Error("unknown webpack runtime function"); };
var REQUIRE_CALLEE = types_1.identifier("require");
var DEFINEPROPERTY_CALLEE = types_1.memberExpression(types_1.identifier("Object"), types_1.identifier("defineProperty")); // Object.defineProperty
var WEBPACK_REQUIRE_N_CALLEE = types_1.identifier("__webpack_getDefaultExport");
var WEBPACK_REQUIRE_N = types_1.toStatement(parser_1.parseExpression("\n/**\n * @template T\n * @argument {T} m\n * @returns {T extends { __esModule: true } ? () => T['default'] : () => T}\n */\nfunction " + WEBPACK_REQUIRE_N_CALLEE.name + "(m) {\n    var getter = m && m.__esModule ?\n        function getDefault() { return m[\"default\"] } :\n        function getModuleExports() { return m }\n        Object.defineProperty(getter, \"a\", { enumerable: true, get: getter })\n    return getter\n}\n"));
var WEBPACK_REQUIRE_E_CALLEE = types_1.identifier("__webpack_loadChunk");
var WEBPACK_REQUIRE_E = types_1.toStatement(parser_1.parseExpression("\nfunction " + WEBPACK_REQUIRE_E_CALLEE.name + "(chunkId) {\n    return Promise.resolve(void chunkId) // noop\n}\n"));
var addGlobalStatement = function (path, statement) {
    var rootNode = path.scope.getProgramParent().path.node;
    if (!rootNode.body.includes(statement)) {
        rootNode.body.push(statement);
    }
};
var isWebpackRequire = function (node) {
    return types_1.isIdentifier(node, { name: "__webpack_require__" });
};
var transformWebpackRequire = function (path, requireIdE, entryId) {
    if (!types_1.isNumericLiteral(requireIdE) && !types_1.isStringLiteral(requireIdE)) {
        throw transformErr();
    }
    var requireId = requireIdE.value;
    var isEntryRequire = requireId == entryId;
    var requirePath = "./" + (isEntryRequire ? "entry_" : "") + requireId;
    path.replaceWith(types_1.callExpression(REQUIRE_CALLEE, [types_1.stringLiteral(requirePath)]));
};
/**
 * Transform variables defined in ModuleFunction params to their real values
 *
 * minified ModuleFunction: `function(e,t,n){`,
 * unminified: `function(module, exports, __webpack_require__) {`.
 *
 * replace `e` with `module`, `t` with `exports`, `n` -> `__webpack_require__` in extracted program body,
 * and transform `__webpack_require__` to normal `require`
 */
exports.getModuleFunctionParamsTransformer = function (entryId) {
    return visitor_wrapper_1.VisitorWrapper({
        Program: function (path) {
            var node = path.node;
            var paramsTransE = node.body[0];
            if (!types_1.isVariableDeclaration(paramsTransE, { kind: "const" })) {
                throw transformErr();
            }
            paramsTransE.declarations.forEach(function (d) {
                var id = d.id, init = d.init;
                if (!types_1.isIdentifier(id) || !types_1.isIdentifier(init)) {
                    throw transformErr();
                }
                var paramName = id.name;
                var realValue = init.name;
                path.scope.rename(paramName, realValue);
            });
            // remove paramsTransE
            node.body.shift();
        },
        // transform `__webpack_require__`
        CallExpression: function (path) {
            var node = path.node;
            var callee = node.callee, callArgs = node.arguments;
            // check __webpack_require__
            if (isWebpackRequire(callee)) {
                // transform `__webpack_require__` to normal `require`
                // `__webpack_require__(0)` -> `require("./0")`
                var requireIdE = callArgs[0];
                transformWebpackRequire(path, requireIdE, entryId);
            }
            else if ( // __webpack_require__.fn(exports, …
            types_1.isMemberExpression(callee) &&
                isWebpackRequire(callee.object) &&
                types_1.isIdentifier(callee.property)) {
                var method = callee.property.name;
                if (method == "d") { // callArgs.length > 1
                    // transform `__webpack_require__.d(exports, "…", function(){…})`
                    // __webpack_require__.d: define getter function for harmony exports
                    var _a = __read(callArgs, 3), exportName = _a[1], exportGetter = _a[2];
                    if (!types_1.isStringLiteral(exportName) && !types_1.isFunctionExpression(exportGetter)) {
                        throw transformErr();
                    }
                    path.replaceWith(types_1.callExpression(DEFINEPROPERTY_CALLEE, [
                        types_1.identifier("exports"),
                        exportName,
                        types_1.objectExpression([
                            types_1.objectProperty(types_1.identifier("enumerable"), types_1.booleanLiteral(true)),
                            types_1.objectProperty(types_1.identifier("get"), exportGetter),
                        ])
                    ]));
                }
                else if (method == "r") {
                    // transform `__webpack_require__.r(exports)
                    // __webpack_require__.r: define __esModule on exports
                    //                        (useless for our "re-tidied" code)
                    // path.replaceWithSourceString(
                    //      `Object.defineProperty(exports, '__esModule', { value: true });`
                    // )
                    path.replaceWith(types_1.callExpression(DEFINEPROPERTY_CALLEE, [
                        types_1.identifier("exports"),
                        types_1.stringLiteral("__esModule"),
                        types_1.objectExpression([
                            types_1.objectProperty(types_1.identifier("value"), types_1.booleanLiteral(true)),
                        ])
                    ]));
                }
                else if (method == "n") {
                    // __webpack_require__.n(…)
                    // __webpack_require__.n: getDefaultExport function for compatibility with non-harmony modules
                    if (callArgs.length !== 1 || !types_1.isIdentifier(callArgs[0])) {
                        throw transformErr();
                    }
                    path.replaceWith(types_1.callExpression(WEBPACK_REQUIRE_N_CALLEE, callArgs));
                    addGlobalStatement(path, WEBPACK_REQUIRE_N);
                }
                else if (method == "bind") {
                    // __webpack_require__.bind(null, …)
                    if (!types_1.isNullLiteral(callArgs[0])) {
                        throw transformErr();
                    }
                    var requireIdE = callArgs[1];
                    transformWebpackRequire(path, requireIdE, entryId);
                }
                else if (method == "e") {
                    // __webpack_require__.e: load jsonp chunk
                    var _b = __read(callArgs, 1), chunkId = _b[0];
                    if (callArgs.length !== 1) {
                        throw transformErr();
                    }
                    if (!types_1.isNumericLiteral(chunkId) && !types_1.isStringLiteral(chunkId)) {
                        throw transformErr();
                    }
                    // `Promise.resolve(/* webpack chunk: ${chunkId.value} */)`
                    path.replaceWith(types_1.callExpression(WEBPACK_REQUIRE_E_CALLEE, callArgs));
                    addGlobalStatement(path, WEBPACK_REQUIRE_E);
                }
                else {
                    console.error(unknownRuntimeFnErr());
                }
            }
        },
    });
};
