"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var visitor_wrapper_1 = __importDefault(require("../utils/visitor-wrapper"));
var types_1 = require("@babel/types");
/**
 * `x && a()` -> `if (x) { a() }`
 * `x || a()` -> `if (!x) { a() }`
 * `x ? a() : b()` -> `if (x) { a() } else { b() }`
 * `return x && a()` -> `if (x) { return a() }` etc.
 */
exports.unminifyIfStatements = visitor_wrapper_1.default({
    ReturnStatement: function (path) {
        var returnE = path.node.argument;
        /**
         * `return x ? a() : b()` -> `if (x) { return a() } else { return b() }`
         */
        if (types_1.isConditionalExpression(returnE)) {
            path.replaceWith(types_1.ifStatement(returnE.test, types_1.returnStatement(returnE.consequent), types_1.returnStatement(returnE.alternate)));
            return;
        }
        /**
         * `return x && a()` -> `if (x) { return a() }`
         * `return x || a()` -> `if (!x) { return a() }`
         */
        if (types_1.isLogicalExpression(returnE)) {
            var test = returnE.operator == "||"
                ? types_1.unaryExpression("!", types_1.parenthesizedExpression(returnE.left))
                : returnE.left;
            path.replaceWith(types_1.ifStatement(test, types_1.returnStatement(returnE.right)));
            return;
        }
    },
    ExpressionStatement: function (path) {
        var expression = path.node.expression;
        /**
         * `x ? a() : b()` -> `if (x) { a() } else { b() }`
         */
        if (types_1.isConditionalExpression(expression)) {
            path.replaceWith(types_1.ifStatement(expression.test, types_1.expressionStatement(expression.consequent), types_1.expressionStatement(expression.alternate)));
            return;
        }
        /**
         * `x && a()` -> `if (x) { a() }`
         * `x || a()` -> `if (!x) { a() }`
         */
        if (types_1.isLogicalExpression(expression)) {
            var test = expression.operator == "||"
                ? types_1.unaryExpression("!", types_1.parenthesizedExpression(expression.left))
                : expression.left;
            path.replaceWith(types_1.ifStatement(test, types_1.expressionStatement(expression.right)));
        }
    },
});
exports.default = exports.unminifyIfStatements;
